/*
 * Copyright (c) 2009 by syntevo GmbH. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  o Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  o Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  o Neither the name of syntevo GmbH nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.syntevo.openapi.smartsvn.command.update;

import java.awt.*;
import java.util.*;
import javax.swing.*;

import org.tmatesoft.svn.core.*;
import org.tmatesoft.svn.core.wc.*;

import com.syntevo.openapi.gui.dialog.*;
import com.syntevo.openapi.smartsvn.command.*;
import com.syntevo.openapi.smartsvn.file.*;

/**
 * An IUpdateWorkflow can be used to customize the workflow of the update operation.
 * The workflow of the update operation is separated into a couple of phases:
 * <ul>
 * <li>A configuration phase (that's typically the 'Update' dialog)</li
 * <li>The pre-command phase which is executed after the user has confirmed the
 * 'Update' dialog with 'OK' and directly before the actual update is performed.
 * It allows to skip certain {@link UpdateEntry} objects for the actual update.</li>
 * <li>The post-command phase which is executed after a  <b>successful</b> update.
 * It receives the actually committed files/directories and the corresponding
 * {@link SVNEvent} objects.
 * It can  be used for all kinds of post-processing an update.</li>
 * </ul>
 */
public interface IUpdateWorkflow {

	/**
	 * Initialize the workflow with the specified set of {@link Path}
	 * objects and configuration (e.g. for initializing the configuration component).
	 *
	 * @param directoryMode is true in case the Update is performed for one or more directories.
	 * @param configuration should be initialized with default values which will be passed to the configuration phase.
	 */
	void initialize(Set<Path> paths, boolean directoryMode, UpdateConfiguration configuration);

	/**
	 * Returns whether a configuration dialog should be shown.
	 * If true is returned, #createConfigurationComponent will be called.
	 *
	 * @see #createConfigurationComponent
	 */
	boolean hasConfigurationPhase();

	/**
	 * Creates the UI components for the shown configuration dialog.
	 *
	 * @param parent the parent for the returned component
	 */
	JComponent createConfigurationComponent(Container parent);

	/**
	 * Returns the component which will be focussed when the 'Update' dialog is displayed.
	 */
	JComponent getConfigurationFocusableComponent();

	/**
	 * Fills the specified configuration (e.g. from the user input to the configuration component).
	 * The resulting configuration will be used for the update operation.
	 */
	void getConfiguration(UpdateConfiguration configuration) throws DialogValidationFailedException;

	/**
	 * Called directly before the actual update operation is invoked. The specified set of {@link UpdateEntry}
	 * objects will be update. It may be altered before it's returned.
	 *
	 * @return the set of {@link UpdateEntry} objects which will actually be updated.
	 */
	Set<UpdateEntry> preCommand(Set<UpdateEntry> updateEntries, SvnClients svnClients) throws SVNException;

	/**
	 * Called directly after an successful update operation. Processes the Map of {@link Path} to
	 * {@link SVNEvent} objects.
	 */
	void postCommand(Map<Path, SVNEvent> pathToEvent, SvnClients svnClients) throws SVNException;
}
