package com.syntevo.plugin.jira.commit.workflow;

import java.io.*;
import java.util.*;
import javax.xml.soap.*;

import com.syntevo.openapi.gui.*;
import com.syntevo.openapi.gui.dialog.*;
import com.syntevo.openapi.smartsvn.command.commit.*;
import com.syntevo.openapi.smartsvn.settings.*;
import com.syntevo.openapi.smartsvn.thread.*;
import com.syntevo.plugin.jira.*;
import com.syntevo.plugin.jira.commit.*;
import com.syntevo.plugin.jira.transport.*;

/**
 * @author syntevo GmbH
 */
final class JiraFilesPhase extends FilterCommitPacketPhase {

	// Constants ==============================================================

	private static final boolean RESOLVE_ENABLED = "true".equals(System.getProperty("smartsvn.plugin.jira.show-resolve-dialog", "true"));

	// Fields =================================================================

	private final SettingsServices settingsServices;
	private final JobExecutor jobExecutor;
	private final DialogDisplayer dialogDisplayer;
	private final GuiSpacings spacings;

	private List<JiraResolvePacket> selectedResolvePackets;

	// Setup ==================================================================

	public JiraFilesPhase(ICommitPacketPhase internalPhase, SettingsServices settingsServices, JobExecutor jobExecutor, GuiSpacings spacings, DialogDisplayer dialogDisplayer) {
		super(internalPhase);
		this.settingsServices = settingsServices;
		this.jobExecutor = jobExecutor;
		this.dialogDisplayer = dialogDisplayer;
		this.spacings = spacings;
	}

	// Implemented ============================================================

	public boolean updateTo(CommitPacketPhaseState state) {
		if (!super.updateTo(state)) {
			return false;
		}

		selectedResolvePackets = null;

		if (!RESOLVE_ENABLED) {
			return true;
		}
		
		final Set<JiraResolvablePacket> allResolvablePackets = new HashSet<JiraResolvablePacket>();
		for (CommitPacket packet : new ArrayList<CommitPacket>(state.getPackets())) {
			allResolvablePackets.addAll(extractResolvablePackets(packet, settingsServices, spacings, dialogDisplayer));
		}

		if (allResolvablePackets.isEmpty()) {
			return true;
		}

		final List<JiraResolvablePacket> resolvablePackets = filterAndConfigureResolvablePackets(allResolvablePackets, jobExecutor, dialogDisplayer);
		selectedResolvePackets = chooseResolvePackets(resolvablePackets);
		return selectedResolvePackets != null;
	}

	// Accessing ==============================================================

	public List<JiraResolvePacket> getResolvePackets() {
		return selectedResolvePackets != null ? Collections.unmodifiableList(selectedResolvePackets) : null;
	}

	// Utils ==================================================================

	private List<JiraResolvePacket> chooseResolvePackets(final List<JiraResolvablePacket> resolvablePackets) {
		JiraPlugin.LOGGER.debug("Letting user choose packets to resolve");

		final List<JiraResolvePacket> selectedResolvePackets = new ArrayList<JiraResolvePacket>();
		final boolean[] abort = new boolean[1];
		GuiUtils.executeInEventDispatchThread(new Runnable() {
			public void run() {
				for (final JiraResolvablePacket packet : resolvablePackets) {
					final JiraResolveIssueDialog resolveIssueDialog = JiraResolveIssueDialog.getInstance(packet, spacings);
					if (resolveIssueDialog == null) {
						JiraPlugin.LOGGER.debug(" ... " + packet + " has no unreleased versions");
						continue;
					}

					if (!dialogDisplayer.show(resolveIssueDialog)) {
						JiraPlugin.LOGGER.debug(" ... " + packet + " cancelled");
						abort[0] = true;
						return;
					}

					if (!resolveIssueDialog.isResolve()) {
						JiraPlugin.LOGGER.debug(" ... " + packet + " not selected");
						continue;
					}

					JiraPlugin.LOGGER.debug(" ... " + packet + " selected");
					selectedResolvePackets.add(new JiraResolvePacket(packet.getConnection(), packet.getIssueKey(), resolveIssueDialog.getFixVersion()));
				}
			}
		});
		return !abort[0] ? selectedResolvePackets : null;
	}

	private static boolean isResolvable(JiraIssue issue) {
		final JiraIssueStatus status = issue.getStatus();
		return status == JiraIssueStatus.IN_PROGRESS || status == JiraIssueStatus.OPEN || status == JiraIssueStatus.REOPENED;
	}

	private static Collection<JiraResolvablePacket> extractResolvablePackets(CommitPacket packet, SettingsServices settingsServices, GuiSpacings spacings, DialogDisplayer dialogDisplayer) {
		JiraPlugin.LOGGER.debug("Extracting resolvable packets for " + packet.getCommitRoot());

		final CommitBugtraqProperties bugtraqProperties = packet.getBugtraqProperties();
		if (bugtraqProperties == null) {
			JiraPlugin.LOGGER.debug(" ... no bugtraq-properties");
			return Collections.emptyList();
		}

		final JiraUrl url = JiraUrl.parseFromBugtraqUrls(bugtraqProperties.getURL());
		if (url == null) {
			JiraPlugin.LOGGER.debug(" ... no bugtraq URL");
			return Collections.emptyList();
		}

		JiraPlugin.LOGGER.debug(" ... URL is " + url);

		final String message = packet.getMessage();
		final String projectKey = url.getProjectKey();
		JiraPlugin.LOGGER.debug(" ... projectKey is " + projectKey);

		final List<JiraResolvablePacket> resolvePackets = new ArrayList<JiraResolvablePacket>();
		final JiraUIConnection connection = new JiraUIConnection(url.getBaseUrl(), settingsServices, true, spacings, dialogDisplayer);

		JiraPlugin.LOGGER.debug("Parsing message '" + message + "'");
		for (int index = message.indexOf(projectKey); index != -1; index = message.indexOf(projectKey, index + 1)) {
			final String issueID = extractIssueID(message, projectKey, index);
			if (issueID == null) {
				index++;
				continue;
			}

			JiraPlugin.LOGGER.debug(" ... found issue " + issueID);
			resolvePackets.add(new JiraResolvablePacket(issueID, projectKey, connection));
			index += issueID.length();
		}

		return resolvePackets;
	}

	private static boolean isIssueIDCharacter(char ch) {
		return Character.isLetter(ch) || Character.isDigit(ch) || ch == '-';
	}

	private static String extractIssueID(String message, String projectKey, int startIndex) {
		if (startIndex > 0 && isIssueIDCharacter(message.charAt(startIndex - 1))) {
			return null;
		}

		final int dashIndex = startIndex + projectKey.length();
		if (message.length() <= dashIndex || message.charAt(dashIndex) != '-') {
			return null;
		}

		int index = dashIndex + 1;

		final StringBuilder number = new StringBuilder();
		while (index < message.length() && Character.isDigit(message.charAt(index))) {
			number.append(message.charAt(index));
			index++;
		}

		if (index < message.length() && isIssueIDCharacter(message.charAt(index))) {
			return null;
		}

		return message.substring(startIndex, index);
	}

	private static List<JiraResolvablePacket> filterAndConfigureResolvablePackets(Set<JiraResolvablePacket> allResolvablePackets, JobExecutor jobExecutor, final DialogDisplayer dialogDisplayer) {
		JiraPlugin.LOGGER.debug("Filtering resolvable packets");

		final List<JiraResolvablePacket> resolvablePackets = new ArrayList<JiraResolvablePacket>();
		final List<JiraResolvablePacket> resolvablePacketsInOrder = new ArrayList<JiraResolvablePacket>(allResolvablePackets);
		Collections.sort(resolvablePacketsInOrder, new Comparator<JiraResolvablePacket>() {
			public int compare(JiraResolvablePacket o1, JiraResolvablePacket o2) {
				return o1.getIssueKey().compareTo(o2.getIssueKey());
			}
		});

		jobExecutor.executeWait("JIRA", new IJob() {
			public void execute(ProgressViewer progressViewer) {
				int progress = 0;
				for (final JiraResolvablePacket packet : resolvablePacketsInOrder) {
					progressViewer.setMessage("Querying " + packet.getConnection().getUrl());
					progressViewer.setProgress(progress++ / (double)resolvablePacketsInOrder.size());

					final JiraSoapClient client = new JiraSoapClient(packet.getConnection());
					try {
						if (!client.login()) {
							JiraPlugin.LOGGER.warn("JIRA login failed, can't check " + packet);
							continue;
						}

						try {
							final JiraIssue issue = client.getIssueByKey(packet.getIssueKey());
							if (!isResolvable(issue)) {
								JiraPlugin.LOGGER.debug(" ... " + packet + " not resolvable (status=" + issue.getStatus() + ")");
								continue;
							}

							final List<JiraVersion> unreleasedVersions = client.getUnreleasedVersionsByProjectKeyInOrder(packet.getProjectKey());
							packet.setIssue(issue);
							packet.setUnreleasedVersions(unreleasedVersions);
							resolvablePackets.add(packet);
							JiraPlugin.LOGGER.debug(" ... " + packet + " is resolvable.");
						}
						finally {
							client.logout();
						}
					}
					catch (IOException ex) {
						JiraPlugin.LOGGER.error(ex.getMessage(), ex);
						dialogDisplayer.showErrorDialog("JIRA", ex.getMessage(), null);
					}
					catch (SOAPException ex) {
						JiraPlugin.LOGGER.error(ex.getMessage(), ex);
						dialogDisplayer.showErrorDialog("JIRA", ex.getMessage(), null);
					}
				}
			}
		});

		return resolvablePackets;
	}
}
